# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# SPDX-License-Identifier: Apache-2.0
import setuptools

setuptools.setup(
    python_requires=">=3.8", setup_requires=["pbr>=2.0.0"], pbr=True
)
